

local major, minor, addonTable = LibStub("LibBuilder-1.0"):GetLib()


local Stream, stream = addonTable:NewClass("Stream")


local function debug(...) --[[print(...)]] end


function Stream:New()
	local instance = {}
	setmetatable(instance, self.instanceMetatable)
	instance.destinations = {}
	return instance
end


--	TODO: think of a more appropriate name
function stream:GetActiveDestinations()
	local active = {}
	
	for i, destination in ipairs(self.destinations) do
		debug(type(destination))
		if destination:IsAvailable() then
			table.insert(active, destination)
		end
		
		--	If it's exclusive, don't look for any more
		if destination:IsExclusive() then
			break
		end
	end
	
	--print("active destinations", unpack(active))
	
	return active
end


function stream:Print(...)
	--print("stream:Print", ...)
	
	local message = self:BuildMessage(...)
	
	if not message then
		return
	end
	
	local destinations = self:GetActiveDestinations()
	for i, destination in ipairs(destinations) do
		if destination:IsRaw() then
			--	FIXME: pass color and prefix
			destination:Print({...})
		else
			destination:Print(message)
		end
	end
end


function stream:BuildMessage(...)
	--	the vararg might end with nils, so count them 
	--	before packing
	local argCount = select("#", ...)
	
	if argCount < 1 then
		--	just ignore the call (don't even display prefix)
		return
	end

	local message = ""
	
	local units
	
	if self.prefix and self.prefix ~= "" then
		units = { self.prefix, ... }
		argCount = argCount+1
	else
		units = { ... }
	end

	if self.color then
		message = message.."|c"..self.color
	end
	
	--	print(nil, "s")
	message = message..units[1]
	
	for i=2,argCount do
		message = message .. " "
		
		if self.color then
			message = message .. "|c" .. self.color
		end
		
		message = message .. ( tostring(units[i]) or "nil" )
	end
	
	--	FIXME: don't break coloring after links

	return message
end


function stream:SetPrefix(newPrefix)
	newPrefix = newPrefix or ""

	assert(type(newPrefix) == "string")
	
	self.prefix = newPrefix
end


function stream:SetColor(newColor)
	assert(newColor == nil or type(newColor) == "string")
	
	--	FIXME: make sure it has the proper format
	
	self.color = newColor
end



function stream:ClearDestinations()
	self.destinations = {}
end


function stream:RemoveDestination(destination)
	local index = 1
	
	while index <= table.maxn(self.destinations) do
		local temp = self.destinations[index]
		
		if temp.value == destination.value and 
			temp.type == destination.type then
			self.destinations[index] = nil
			break
		end
	end
	
	--	now shift everything down
	while index <= table.maxn(self.destinations) do
		self.destinations[index] = self.destinations[index+1]
	end
end


function stream:AddDestination(newDestination, position)
	if not position then
		position = table.maxn(self.destinations) + 1
	end
	--	FIXME: make sure it's new
	--	FIXME: make sure position is valid
	
	for i = table.maxn(self.destinations), position, -1 do
		self.destinations[i+1] = self.destinations[i]
	end
	
	self.destinations[position] = newDestination
end

	

function Stream:ApplyColor(message, color)
end
